#!/usr/local/miniconda2/bin/python

"""
================================
Detecting caverns with SVM
================================

"""
print(__doc__)

import sys, os
import sepbase as sep
import numpy as np
from sklearn import svm
from sklearn import metrics
from sklearn import preprocessing
from sklearn.decomposition import PCA

from time import time
import matplotlib.pyplot as plt
from matplotlib import cm, colors

def main(argc, argv):

	hfile_training_data   = str(sys.argv[1])
	hfile_training_labels = str(sys.argv[2])
	hfile_test_data       = str(sys.argv[3])
	hfile_test_labels     = str(sys.argv[4])
	hfile_output          = str(sys.argv[5])

	# TRAINING DATA ---------------------------------------

	print("------------------------------------------------")

	print(">> Reading training data")
	n1 = int(sep.get_sep_his_par(hfile_training_data, 'n1'))
	n2 = int(sep.get_sep_his_par(hfile_training_data, 'n2'))
	n3 = int(sep.get_sep_his_par(hfile_training_data, 'n3'))

	dfile_tmp = sep.get_sep_his_par(hfile_training_data, 'in')
	training_data  = np.fromfile(dfile_tmp, dtype=np.float32)
	training_data  = training_data.byteswap()
	training_data  = training_data.reshape(n3, n1*n2).T
	print "Training data shape                       :", np.shape(training_data)

	# TRAINING LABELS -------------------------------------

	dfile_tmp = sep.get_sep_his_par(hfile_training_labels, 'in')
	training_labels = np.fromfile(dfile_tmp, dtype=np.float32)
	training_labels = training_labels.byteswap()
	training_labels = training_labels.reshape(n1*n2)
	training_labels = training_labels.astype(int)

	print "Number of cavern samples in training data :", training_labels.sum()
	print "Training labels shape                     :", np.shape(training_labels)

	# TEST DATA -------------------------------------------

	print(">> Reading test data")
	n1 = int(sep.get_sep_his_par(hfile_test_data, 'n1'))
	n2 = int(sep.get_sep_his_par(hfile_test_data, 'n2'))
	n3 = int(sep.get_sep_his_par(hfile_test_data, 'n3'))

	dfile_tmp = sep.get_sep_his_par(hfile_test_data, 'in')
	test_data = np.fromfile(dfile_tmp, dtype=np.float32)
	test_data = test_data.byteswap()
	test_data = test_data.reshape(n3, n1*n2).T
	print "Test data shape                           :", np.shape(test_data)

	# TEST LABELS -----------------------------------------

	dfile_tmp = sep.get_sep_his_par(hfile_test_labels, 'in')
	test_labels = np.fromfile(dfile_tmp, dtype=np.float32)
	test_labels = test_labels.byteswap()
	test_labels = test_labels.reshape(n1*n2)
	test_labels = test_labels.astype(int)
	print "Number of cavern samples in test data     :", test_labels.sum()
	print "Test labels shape                         :", np.shape(test_labels)

	# PREPROCESSING ---------------------------------------

	training_data = training_data.astype(np.float64)
	test_data = test_data.astype(np.float64)

	# STANDARD SCALER 
	scaler = preprocessing.StandardScaler().fit(training_data)
	training_data = scaler.transform(training_data)
	test_data = scaler.transform(test_data)


	print "C_contiguous array check :", training_data.flags['C_CONTIGUOUS']

	# SANITY CHECK ----------------------------------------

	# fig = plt.figure()
	# plt.imshow(training_data[:,0])
	# plt.show()

	# fig = plt.figure()
	# plt.plot(training_data[:,1])
	# plt.show()

	# fig = plt.figure()
	# plt.plot(training_data[:,2])
	# plt.show()

	# reduced_data = PCA(n_components=2).fit_transform(training_data)

	# jet    = plt.get_cmap('jet') 
	# cNorm  = colors.Normalize(vmin=0, vmax=1)
	# scalarMap = cm.ScalarMappable(norm=cNorm, cmap=jet)
	# plt.figure()
	# plt.scatter(training_data[:, 0], training_data[:, 1],
 #            marker='.',
 #            color=scalarMap.to_rgba(training_labels))
	# plt.show()

	# CLASSIFIER ------------------------------------------

	# (C=1.0, kernel='rbf', degree=3, gamma='auto', coef0=0.0, shrinking=True, probability=False, 
	#	tol=0.001, cache_size=200, class_weight=None, verbose=False, max_iter=-1, decision_function_shape=None, 
	# random_state=None)

	print("------------------------------------------------")

	print(">> Creating classifier")
	classifier = svm.SVC(C=1, kernel='rbf', gamma=1000., cache_size=2000, class_weight='balanced', verbose=False)
	print("Fitting classifier on training data")
	t0 = time()
	classifier.fit(training_data, training_labels)
	print "Fitting on training data :", time()-t0, "s"

	# print("Predicting training data")
	# expected  = training_labels
	# predicted = classifier.predict(training_data)

	# print("------------------------------------------------")
	# print("Training report for classifier %s:\n%s\n"
 #      % (classifier, metrics.classification_report(expected, predicted)))
	# print("Confusion matrix:\n%s" % metrics.confusion_matrix(expected, predicted))

	expected  = test_labels
	t0 = time()
	print("Predicting test data")
	predicted = classifier.predict(test_data)
	print "Prediction on test data :", time()-t0, "s"

	print("------------------------------------------------")
	print("Classification report for classifier %s:\n%s\n"
      % (classifier, metrics.classification_report(expected, predicted)))
	print("Confusion matrix:\n%s" % metrics.confusion_matrix(expected, predicted))
	print("------------------------------------------------")


	# WRITE OUTPUT ----------------------------------------

	predicted = predicted.astype(np.float32)
	predicted = predicted.byteswap()

	sep.put_sep_axis_params(hfile_output, 1, [n1, 0.0, 4.0, ''])
	sep.put_sep_axis_params(hfile_output, 2, [n2, 0.0, 4.0, ''])

	cmd = "echo in=%s >> %s" % (hfile_output+'@', hfile_output)
	os.system(cmd)

	with open (hfile_output+'@', 'wb') as f:
		predicted.tofile(f)
		f.close()

if __name__ == "__main__":
	main (len(sys.argv), sys.argv)
