! Main program to combine windowed images into a larger image
program combine

  use sep

  implicit none

  integer                               :: nz,nx,iz,ix,stat,sseek,auxinout
  integer								:: xstart,zstart,abc,nxT,nzT
  real                                  :: dz,oz,dx,ox
  real, allocatable, dimension(:,:)     :: lambda_in,mu_in,rho_in
  real, allocatable, dimension(:,:)     :: lambda_out,mu_out,rho_out
  logical                               :: add

  call sep_init()

  ! Model parameters
  ! Dimensions
  call from_history("n1",nz)                    !# of samples in depth
  call from_history("o1",oz,0.0)                !origin for depth coord
  call from_history("d1",dz)                    !grid spacing in depth
  call from_history("n2",nx)                    !# of samples in x
  call from_history("o2",ox,0.0)                !origin for x coord
  call from_history("d2",dx)                    !grid spacing in x

  call from_param("nxtotal",nxT)   	   			!number os samples of total model in X
  call from_param("nztotal",nzT)	      		!number os samples of total model in Z
  call from_param("abc",abc)			   		    !absorbing boundary to be ignored
  call from_param("xstart",xstart)			   	!start of window in x
  call from_param("zstart",zstart)			   	!start of window in z
  call from_param("add",add)                    !wheter to create a newfile for output or not

  write(0,*)"=============================================="
  write(0,*)"Model size (z,x)",nzT,nxT
  write(0,*)"Absorbing boundary",abc
  write(0,*)"Start of model window (z,x)",zstart,xstart
  write(0,*)"Add=",add
  write(0,*)"=============================================="

  allocate(lambda_in(nz,nx))
  allocate(mu_in(nz,nx))
  allocate(rho_in(nz,nx))
  allocate(lambda_out(nzT,nxT))
  allocate(mu_out(nzT,nxT))
  allocate(rho_out(nzT,nxT))

  call sep_read(lambda_in)
  call sep_read(mu_in,"mu_in")
  call sep_read(rho_in,"rho_in")

  if (add) then
    stat = auxinout("lambda_out")
    call sep_read(lambda_out,"lambda_out")
    stat = auxinout("mu_out")
    call sep_read(mu_out,"mu_out")
    stat = auxinout("rho_out")
    call sep_read(rho_out,"rho_out")
  else
    lambda_out=0.0
    mu_out    =0.0
    rho_out   =0.0
  endif

  do ix=1,nx-2*abc
    do iz=1,nz-2*abc
      lambda_out(zstart+iz-1,xstart+ix-1) = lambda_out (zstart+iz-1,xstart+ix-1) + lambda_in (abc+iz,abc+ix)
      mu_out    (zstart+iz-1,xstart+ix-1) = mu_out     (zstart+iz-1,xstart+ix-1) + mu_in     (abc+iz,abc+ix)
      rho_out   (zstart+iz-1,xstart+ix-1) = rho_out    (zstart+iz-1,xstart+ix-1) + rho_in    (abc+iz,abc+ix)
    enddo
  enddo

  if (add) then
    stat = sseek("lambda_out",0,0) !first zero means how many samples to move (put *4), second means where to begin the move,
    stat = sseek("mu_out",0,0)     !with zero meaning file start and 1 meaning current position
    stat = sseek("rho_out",0,0)
  endif

  call to_history("n1",nzT,"lambda_out")      !# of samples in depth
  call to_history("o1",oz,"lambda_out")       !origin for depth coord
  call to_history("d1",dz,"lambda_out")       !grid spacing in depth
  call to_history("n2",nxT,"lambda_out")      !# of samples in x
  call to_history("o2",0.0,"lambda_out")      !origin for x coord
  call to_history("d2",dx,"lambda_out")       !grid spacing in x

  call to_history("n1",nzT,"mu_out")          !# of samples in depth
  call to_history("o1",oz,"mu_out")           !origin for depth coord
  call to_history("d1",dz,"mu_out")           !grid spacing in depth
  call to_history("n2",nxT,"mu_out")          !# of samples in x
  call to_history("o2",0.0,"mu_out")           !origin for x coord
  call to_history("d2",dx,"mu_out")           !grid spacing in x

  call to_history("n1",nzT,"rho_out")         !# of samples in depth
  call to_history("o1",oz,"rho_out")          !origin for depth coord
  call to_history("d1",dz,"rho_out")          !grid spacing in depth
  call to_history("n2",nxT,"rho_out")         !# of samples in x
  call to_history("o2",0.0,"rho_out")          !origin for x coord
  call to_history("d2",dx,"rho_out")          !grid spacing in x

  call sep_write(lambda_out,"lambda_out")
  call sep_write(mu_out,"mu_out")
  call sep_write(rho_out,"rho_out")

  deallocate(lambda_in,mu_in,rho_in,lambda_out,mu_out,rho_out)

  call sep_close()

end program
