! Main program to create 2D models
program convert_model_moerevest

  use sep

  implicit none

  integer                               :: nt,nx,ix,it,it2
  real                                  :: dt,ot,dx,ox
  real, allocatable, dimension(:,:)     :: lambda, mu, rho, lambda_out, mu_out, rho_out

  call sep_init()

  ! Model parameters
  ! Dimensions
  call from_history("n1",nt)                    !# of samples in depth
  call from_history("o1",ot,0.0)                !origin for depth coord
  call from_history("d1",dt)                    !grid spacing in depth
  call from_history("n2",nx)                    !# of samples in x
  call from_history("o2",ox,0.0)                !origin for x coord
  call from_history("d2",dx)                    !grid spacing in x

  ! Write SEP header parameter
  call to_history("n1",nz)
  call to_history("o1",oz)
  call to_history("d1",dz)
  call to_history("n2",nx)
  call to_history("o2",ox)
  call to_history("d2",dx)
  call to_history("label1","z")
  call to_history("label2","x")
  call to_history("n1",nz,"rho_out")
  call to_history("o1",oz,"rho_out")
  call to_history("d1",dz,"rho_out")
  call to_history("n2",nx,"rho_out")
  call to_history("o2",ox,"rho_out")
  call to_history("d2",dx,"rho_out")
  call to_history("label1","z","rho_out")
  call to_history("label2","x","rho_out")
  call to_history("n1",nz,"mu_out")
  call to_history("o1",oz,"mu_out")
  call to_history("d1",dz,"mu_out")
  call to_history("n2",nx,"mu_out")
  call to_history("o2",ox,"mu_out")
  call to_history("d2",dx,"mu_out")
  call to_history("label1","z","mu_out")
  call to_history("label2","x","mu_out")

  write(0,*) "ALLOCATION => OK"
  
  ! Initialize values
  lambda=0.
  mu=0.
  rho=0.

  write(0,*) "INITIALIZE VALUES => OK"

  call sep_read(lambda)
  call sep_read(mu,"mu")
  call sep_read(rho,"rho")

  ! Allocation of the input/output matrices
  allocate(lambda(nz,nx))
  allocate(mu(nz,nx))
  allocate(rho(nz,nx))

  write(0,*) "READ => OK"

  do ix=1,nx
    it=1
    do while (temp(it,ix).eq.1)
    	lambda(it,ix) = 2.25E3
		mu(it,ix) = 0.0
		rho(it,ix) = 1.0E3
		it = it + 1
	enddo
	do it2=it,nt
	    lambda(it2,ix) = 4.1E3 + 31*(it2-it) 
	    mu(it2,ix) = 170.0 + 31*(it2-it)
	    rho(it2,ix) = 2.0E3 + 1.0*(it2-it)
	enddo
  enddo

  write(0,*) "KERNEL => OK"
  
  call sep_write(lambda)
  call sep_write(mu_out,"mu_out")
  call sep_write(rho_out,"rho_out")
  
  write(0,*) "WRITE => OK"

  call sep_close()

end program
  