! Main program to add a maks to the top of the model so that model(smooth) = model(true) at source
program create_s_lame

  use sep

  implicit none

  integer                               :: nz,nx,iz,ix,z0
  real                                  :: dz,oz,dx,ox
  real                                  :: aux,alfa,vz
  real, allocatable, dimension(:,:)     :: lambda_in,mu_in,rho_in
  real, allocatable, dimension(:,:)     :: lambda_out,mu_out,rho_out
  integer,allocatable, dimension(:)     :: water_bt

  call sep_init()

  ! Model parameters
  ! Dimensions
  call from_history("n1",nz)                    !# of samples in depth
  call from_history("o1",oz,0.0)                !origin for depth coord
  call from_history("d1",dz)                    !grid spacing in depth
  call from_history("n2",nx)                    !# of samples in x
  call from_history("o2",ox,0.0)                !origin for x coord
  call from_history("d2",dx)                    !grid spacing in x

  call from_param("vz",vz,0.0)              !number os samples of mask

  allocate(lambda_in(nz,nx))
  allocate(mu_in(nz,nx))
  allocate(rho_in(nz,nx))
  allocate(lambda_out(nz,nx))
  allocate(mu_out(nz,nx))
  allocate(rho_out(nz,nx))

  allocate(water_bt(nx))

  call sep_read(lambda_in)
  call sep_read(mu_in,"mu_in")
  call sep_read(rho_in,"rho_in")

  water_bt=0

  do ix=1,nx
    iz=1
    do while (mu_in(iz,ix).eq.0.)
      water_bt(ix)=iz
      iz=iz+1
    enddo
  enddo

  !write(0,*)water_bt

  do ix=1,nx
    z0=water_bt(ix)
    do iz=1,z0
        lambda_out(iz,ix) = lambda_in(iz,ix)
        mu_out(iz,ix)     = mu_in(iz,ix)
        rho_out(iz,ix)    = rho_in(iz,ix)
    end do
    do iz=z0+1,nz
        lambda_out(iz,ix) = lambda_in(z0+1,ix)+lambda_in(z0+1,ix)*(iz-z0)*vz
        mu_out(iz,ix)     = mu_in(z0+1,ix)+mu_in(z0+1,ix)*(iz-z0)*vz
        !rho_out(iz,ix)    = rho_in(z0+1,ix)*(iz-z0)*vz
        rho_out(iz,ix)    = rho_in(z0+1,ix)
    enddo
  enddo

  ! Write SEP header parameter
  call to_history("n1",nz)
  call to_history("o1",oz)
  call to_history("d1",dz)
  call to_history("n2",nx)
  call to_history("o2",ox)
  call to_history("d2",dx)
  call to_history("label1","z")
  call to_history("label2","x")

  call sep_write(lambda_out)

  ! Write SEP header parameter
  call to_history("n1",nz,"mu_out")
  call to_history("o1",oz,"mu_out")
  call to_history("d1",dz,"mu_out")
  call to_history("n2",nx,"mu_out")
  call to_history("o2",ox,"mu_out")
  call to_history("d2",dx,"mu_out")
  call to_history("label1","z","mu_out")
  call to_history("label2","x","mu_out")

  call sep_write(mu_out,"mu_out")

  ! Write SEP header parameter
  call to_history("n1",nz,"rho_out")
  call to_history("o1",oz,"rho_out")
  call to_history("d1",dz,"rho_out")
  call to_history("n2",nx,"rho_out")
  call to_history("o2",ox,"rho_out")
  call to_history("d2",dx,"rho_out")
  call to_history("label1","z","rho_out")
  call to_history("label2","x","rho_out")

  call sep_write(rho_out,"rho_out")

  call sep_close

end program
