! Program created by Gustavo Catao Alves at
! the Stanford Exploration Project
! May, 6th, 2014

! seismogram module
module image_mod_rtm

  use sep
  use snap_mod_rtm
  !use omp_lib

  implicit none
  integer, private :: nx,nz,nt,ntseis,ix,iz,itseis
  real, private    :: dx,dz,dt,dtseis
  real, private    :: c1=35.0/294912.0,c2=-405.0/229376.0,c3=567.0/40960.0,c4=-735.0/8192.0,c5=19845.0/16384.0
  real, dimension(:,:,:), pointer, private   :: Vx0new,Vz0new,Vx0cur,Vz0cur
  real, dimension(:,:), pointer, private     :: m1,m2,m3
  real, allocatable, dimension(:,:), private :: S1,S2,S3,S4,S5,S6
  logical, private :: lame

  contains

  ! Initialization of all variables needed by this subroutine
  subroutine image_init(nx_in,dx_in,nz_in,dz_in,nt_in,dt_in,ntseis_in,dtseis_in,Vx0new_in,Vz0new_in,Vx0cur_in,Vz0cur_in,lame_in)
    integer,intent(in)              :: nx_in,nz_in,nt_in,ntseis_in
    real,intent(in)                 :: dx_in,dz_in,dt_in,dtseis_in
    logical                         :: lame_in
    real,dimension(:,:,:),target    :: Vx0new_in,Vz0new_in,Vx0cur_in,Vz0cur_in

    nx = nx_in
    dx = dx_in
    nz = nz_in
    dz = dz_in
    nt = nt_in
    dt = dt_in
    ntseis = ntseis_in
    dtseis = dtseis_in
    lame = lame_in

    Vx0new => Vx0new_in
    Vz0new => Vz0new_in
    Vx0cur => Vx0cur_in
    Vz0cur => Vz0cur_in

    allocate(S1(nz,nx))
    allocate(S2(nz,nx))
    allocate(S3(nz,nx))
    allocate(S4(nz,nx))
    allocate(S5(nz,nx))
    allocate(S6(nz,nx))

    ! Write outputs to the terminal
    write(0,*)"============================================================="
    write(0,*)"MEMORY ALLOCATION FOR IMAGING ARRAYS"
    write(0,*)"============================================================="
    write(0,*)"S1",size(S1)*4/1024,"KB"
    write(0,*)"S2",size(S2)*4/1024,"KB"
    write(0,*)"S3",size(S3)*4/1024,"KB"
    write(0,*)"S4",size(S4)*4/1024,"KB"
    write(0,*)"S5",size(S5)*4/1024,"KB"
    write(0,*)"S6",size(S6)*4/1024,"KB"
    write(0,*)"============================================================="

    S1 = 0.
    S2 = 0.
    S3 = 0.
    S4 = 0.
    S5 = 0.
    S6 = 0.

!     call to_history("n1", nz, "dat/s1.H")
!     call to_history("o1", 0., "dat/s1.H")
!     call to_history("d1", dz, "dat/s1.H")
!     call to_history("n2", nx, "dat/s1.H")
!     call to_history("o2", 0., "dat/s1.H")
!     call to_history("d2", dx, "dat/s1.H")
!     call to_history("n3", ntseis, "dat/s1.H")
!     call to_history("o3", 0., "dat/s1.H")
!     call to_history("d3", dtseis, "dat/s1.H")
! 
!     call to_history("n1", nz, "dat/s2.H")
!     call to_history("o1", 0., "dat/s2.H")
!     call to_history("d1", dz, "dat/s2.H")
!     call to_history("n2", nx, "dat/s2.H")
!     call to_history("o2", 0., "dat/s2.H")
!     call to_history("d2", dx, "dat/s2.H")
!     call to_history("n3", ntseis, "dat/s2.H")
!     call to_history("o3", 0., "dat/s2.H")
!     call to_history("d3", dtseis, "dat/s2.H")
!     
!     call to_history("n1", nz, "dat/s3.H")
!     call to_history("o1", 0., "dat/s3.H")
!     call to_history("d1", dz, "dat/s3.H")
!     call to_history("n2", nx, "dat/s3.H")
!     call to_history("o2", 0., "dat/s3.H")
!     call to_history("d2", dx, "dat/s3.H")
!     call to_history("n3", ntseis, "dat/s3.H")
!     call to_history("o3", 0., "dat/s3.H")
!     call to_history("d3", dtseis, "dat/s3.H")
! 
!     call to_history("n1", nz, "dat/s4.H")
!     call to_history("o1", 0., "dat/s4.H")
!     call to_history("d1", dz, "dat/s4.H")
!     call to_history("n2", nx, "dat/s4.H")
!     call to_history("o2", 0., "dat/s4.H")
!     call to_history("d2", dx, "dat/s4.H")
!     call to_history("n3", ntseis, "dat/s4.H")
!     call to_history("o3", 0., "dat/s4.H")
!     call to_history("d3", dtseis, "dat/s4.H")
! 
!     call to_history("n1", nz, "dat/s5.H")
!     call to_history("o1", 0., "dat/s5.H")
!     call to_history("d1", dz, "dat/s5.H")
!     call to_history("n2", nx, "dat/s5.H")
!     call to_history("o2", 0., "dat/s5.H")
!     call to_history("d2", dx, "dat/s5.H")
!     call to_history("n3", ntseis, "dat/s5.H")
!     call to_history("o3", 0., "dat/s5.H")
!     call to_history("d3", dtseis, "dat/s5.H")
!     
!     call to_history("n1", nz, "dat/s6.H")
!     call to_history("o1", 0., "dat/s6.H")
!     call to_history("d1", dz, "dat/s6.H")
!     call to_history("n2", nx, "dat/s6.H")
!     call to_history("o2", 0., "dat/s6.H")
!     call to_history("d2", dx, "dat/s6.H")
!     call to_history("n3", ntseis, "dat/s6.H")
!     call to_history("o3", 0., "dat/s6.H")
!     call to_history("d3", dtseis, "dat/s6.H")

  endsubroutine
    
  subroutine image_init2(m1_in,m2_in,m3_in)
    real,dimension(:,:),target  :: m1_in,m2_in,m3_in

    m1 => m1_in
    m2 => m2_in
    m3 => m3_in

  endsubroutine

!==================================
! FUNCTION CALLS
!==================================

  function image_op(adj,add,dm1,dm1_x,dm1_z,dm2,dm3,Vx,Vz,sigmaxx,sigmazz,sigmaxz,it) result(stat)
    logical,intent(in) :: adj,add
    real,dimension(:)  :: dm1,dm1_x,dm1_z,dm2,dm3,Vx,Vz,sigmaxx,sigmazz,sigmaxz
    integer            :: stat,it

    call image_op2(adj,add,dm1,dm1_x,dm1_z,dm2,dm3,Vx,Vz,sigmaxx,sigmazz,sigmaxz,it)
    stat=0
  endfunction

  function funddt(adj,add,model1,model2,data) result(stat)
    logical,intent(in) :: adj,add
    real,dimension(:)  :: model1,model2,data
    integer            :: stat
    call ddt(adj,add,model1,model2,data)
    stat=0
  endfunction

  function funvxdx(adj,add,model,data) result(stat)
    logical,intent(in) :: adj,add
    real,dimension(:)  :: model,data
    integer            :: stat
    call vxdx(adj,add,model,data)
    stat=0
  endfunction

  function funvzdz(adj,add,model,data) result(stat)
    logical,intent(in) :: adj,add
    real,dimension(:)  :: model,data
    integer            :: stat
    call vzdz(adj,add,model,data)
    stat=0
  endfunction

  function funvxdz(adj,add,model,data) result(stat)
    logical,intent(in) :: adj,add
    real,dimension(:)  :: model,data
    integer            :: stat
    call vxdz(adj,add,model,data)
    stat=0
  endfunction

  function funvzdx(adj,add,model,data) result(stat)
    logical,intent(in) :: adj,add
    real,dimension(:)  :: model,data
    integer            :: stat
    call vzdx(adj,add,model,data)
    stat=0
  endfunction

!==================================
! SUBROUTINES
!==================================
   subroutine image_op2(adj,add,dm1,dm1_x,dm1_z,dm2,dm3,Vx,Vz,sigmaxx,sigmazz,sigmaxz,it)
    logical,intent(in)                     :: adj,add
    integer                                :: it
    real,dimension(nz,nx),intent(inout)    :: dm1,dm1_x,dm1_z,dm2,dm3
    real,dimension(nz,nx),intent(inout)    :: Vx,Vz,sigmaxx,sigmazz,sigmaxz
    
    integer                          :: wstat,srite

    itseis = it*(dt/dtseis)+1

    call ddt(.false.,.false.,Vx0cur(:,:,itseis),Vx0new(:,:,itseis),S1)
    call ddt(.false.,.false.,Vz0cur(:,:,itseis),Vz0new(:,:,itseis),S2)
    call vxdx(.false.,.false.,Vx0cur(:,:,itseis),S4)
    call vzdz(.false.,.false.,Vz0cur(:,:,itseis),S5)
    S3=S4+S5
    call vxdz(.false.,.false.,Vx0cur(:,:,itseis),S6)
    call vzdx(.false.,.true.,Vz0cur(:,:,itseis),S6)

!       wstat = srite("dat/s1.H",S1,4*nz*nx)
!       wstat = srite("dat/s2.H",S2,4*nz*nx)
!       wstat = srite("dat/s3.H",S3,4*nz*nx)
!       wstat = srite("dat/s4.H",S4,4*nz*nx)
!       wstat = srite("dat/s5.H",S5,4*nz*nx)
!       wstat = srite("dat/s6.H",S6,4*nz*nx)

! IMAGING CONDITION FOR LAME PARAMETERS
    if (lame) then
      if(adj)then
        if(.not.add) then
          dm1_x = 0.
          dm1_z = 0.
          dm2   = 0.
          dm3   = 0.
        endif
        dm1_x(:,:) = dm1_x(:,:) + (Vx(:,:)*S1(:,:))
        dm1_z(:,:) = dm1_z(:,:) + (Vz(:,:)*S2(:,:))
        dm2(:,:) = dm2(:,:) - (sigmaxx(:,:)*S3(:,:)) &
                            - (sigmazz(:,:)*S3(:,:))
        dm3(:,:) = dm3(:,:) - (sigmaxx(:,:)*2.0*S4(:,:)) &
                            - (sigmazz(:,:)*2.0*S5(:,:)) &
                            - (sigmaxz(:,:)*S6(:,:))
      else
        if(.not.add) then
          Vx = 0.
          Vz = 0.
          sigmaxx = 0.
          sigmazz = 0.
          sigmaxz = 0.
        endif
        Vx(:,:) = Vx(:,:) + (dm1_x(:,:)*S1(:,:))
        Vz(:,:) = Vz(:,:) + (dm1_z(:,:)*S2(:,:))
        sigmaxx(:,:) = sigmaxx(:,:) - (dm2(:,:)*S3(:,:)) &
                                    - (dm3(:,:)*2.0*S4(:,:))
        sigmazz(:,:) = sigmazz(:,:) - (dm2(:,:)*S3(:,:)) &
                                    - (dm3(:,:)*2.0*S5(:,:))
        sigmaxz(:,:) = sigmaxz(:,:) - (dm3(:,:)*S6(:,:))
      endif
! IMAGING CONDITION FOR VELOCITY PARAMETERS
    else
      if(adj)then
        if(.not.add) then
          dm1 = 0.
          dm1_x = 0.
          dm1_z = 0.
          dm2 = 0.
          dm3 = 0.
        endif
        dm1_x(:,:) = dm1_x(:,:) + (Vx(:,:)*S1(:,:))
        dm1_z(:,:) = dm1_z(:,:) + (Vz(:,:)*S2(:,:))
        dm1(:,:) = dm1(:,:) - (sigmaxx(:,:)*(m2(:,:)*m2(:,:)*S4(:,:) &
                                           +(m2(:,:)*m2(:,:)-2.0*m3(:,:)*m3(:,:))*S5(:,:))) &
                            - (sigmazz(:,:)*(m2(:,:)*m2(:,:)*S5(:,:) &
                                           +(m2(:,:)*m2(:,:)-2.0*m3(:,:)*m3(:,:))*S4(:,:))) &
                            - (sigmaxz(:,:)*(m3(:,:)*m3(:,:)*S6(:,:)))
        dm2(:,:) = dm2(:,:) - (sigmaxx(:,:)*(2.0*m1(:,:)*m2(:,:)*S3(:,:))) &
                            - (sigmazz(:,:)*(2.0*m1(:,:)*m2(:,:)*S3(:,:)))
        dm3(:,:) = dm3(:,:) + (sigmaxx(:,:)*(4.0*m1(:,:)*m3(:,:)*S5(:,:))) &
                            + (sigmazz(:,:)*(4.0*m1(:,:)*m3(:,:)*S4(:,:))) &
                            - (sigmaxz(:,:)*(2.0*m1(:,:)*m3(:,:)*S6(:,:)))
      else
        if(.not.add) then
          Vx = 0.
          Vz = 0.
          sigmaxx = 0.
          sigmazz = 0.
          sigmaxz = 0.
        endif
        Vx(:,:) = Vx(:,:) + (dm1_x(:,:)*S1(:,:))
        Vz(:,:) = Vz(:,:) + (dm1_z(:,:)*S2(:,:))
        sigmaxx(:,:) = sigmaxx(:,:) - (dm1(:,:)*(m2(:,:)*m2(:,:)*S4(:,:) &
                                               +(m2(:,:)*m2(:,:)-2.0*m3(:,:)*m3(:,:))*S5(:,:))) &
                                     - (dm2(:,:)*(2.0*m1(:,:)*m2(:,:)*S3(:,:))) &
                                     + (dm3(:,:)*(4.0*m1(:,:)*m3(:,:)*S5(:,:)))
        sigmazz(:,:) = sigmazz(:,:) - (dm1(:,:)*(m2(:,:)*m2(:,:)*S5(:,:) &
                                               +(m2(:,:)*m2(:,:)-2.0*m3(:,:)*m3(:,:))*S4(:,:))) &
                                    - (dm2(:,:)*(2.0*m1(:,:)*m2(:,:)*S3(:,:))) &
                                    + (dm3(:,:)*(4.0*m1(:,:)*m3(:,:)*S4(:,:)))
        sigmaxz(:,:) = sigmaxz(:,:) - (dm1(:,:)*(m3(:,:)*m3(:,:)*S6(:,:))) &
                                    - (dm3(:,:)*(2.0*m1(:,:)*m3(:,:)*S6(:,:)))
      endif
    endif
    
  endsubroutine
!==================================
! DERIVATIVES
!==================================

  subroutine ddt(adj,add,model1,model2,data)
    real                                 :: aux
    logical,intent(in)                   :: adj,add
    real,dimension(nz,nx),intent(inout)  :: data
    real,dimension(nz,nx),intent(inout)  :: model1,model2

    aux=1.0/dt

    if(adj) then
      !DONT BOTHER PASSING THE DOT PRODUCT TEST
      !YOU WILL NOT NEED IT!
      if(.not.add) then
        model1 = 0.
        model2 = 0.
      endif
      do ix=1,nx
        do iz=1,nz
          model2(iz,ix) = model2(iz,ix) + aux*data(iz,ix)
          model1(iz,ix) = model1(iz,ix) - aux*data(iz,ix)
        enddo
      enddo
    else
      if(.not.add) data = 0.
      do ix=1,nx
        do iz=1,nz
          data(iz,ix) = data(iz,ix) + aux*(model2(iz,ix)-model1(iz,ix))
        enddo
      enddo
    endif
  endsubroutine

  subroutine vxdx(adj,add,model,data)
    real                                  :: aux
    logical,intent(in)                    :: adj,add
    real,dimension(nz,nx),intent(inout) :: model,data

    aux=1.0/dx

    if(adj) then
      if(.not.add) model = 0.
      do ix=5,nx-5
        do iz=5,nz-5
          model(iz,ix+5) = model(iz,ix+5) + c1*data(iz,ix)*aux
          model(iz,ix+4) = model(iz,ix+4) + c2*data(iz,ix)*aux
          model(iz,ix+3) = model(iz,ix+3) + c3*data(iz,ix)*aux
          model(iz,ix+2) = model(iz,ix+2) + c4*data(iz,ix)*aux
          model(iz,ix+1) = model(iz,ix+1) + c5*data(iz,ix)*aux
          model(iz,ix  ) = model(iz,ix  ) - c5*data(iz,ix)*aux
          model(iz,ix-1) = model(iz,ix-1) - c4*data(iz,ix)*aux
          model(iz,ix-2) = model(iz,ix-2) - c3*data(iz,ix)*aux
          model(iz,ix-3) = model(iz,ix-3) - c2*data(iz,ix)*aux
          model(iz,ix-4) = model(iz,ix-4) - c1*data(iz,ix)*aux
        enddo
      enddo
    else
      if(.not.add) data = 0.
      do ix=5,nx-5
        do iz=5,nz-5
          data(iz,ix) = data(iz,ix) + (c1*(model(iz,ix+5) - model(iz,ix-4))+&
                                       c2*(model(iz,ix+4) - model(iz,ix-3))+&
                                       c3*(model(iz,ix+3) - model(iz,ix-2))+&
                                       c4*(model(iz,ix+2) - model(iz,ix-1))+&
                                       c5*(model(iz,ix+1) - model(iz,ix  )))*aux
        enddo
      enddo
    endif

  endsubroutine

  subroutine vzdz(adj,add,model,data)
    real                                   :: aux
    logical,intent(in)                     :: adj,add
    real,dimension(nz,nx),intent(inout)  :: model,data

    aux=1.0/dx

    if(adj) then
      if(.not.add) model = 0.
      do ix=5,nx-5
        do iz=5,nz-5
          model(iz+5,ix) = model(iz+5,ix) + c1*data(iz,ix)*aux
          model(iz+4,ix) = model(iz+4,ix) + c2*data(iz,ix)*aux
          model(iz+3,ix) = model(iz+3,ix) + c3*data(iz,ix)*aux
          model(iz+2,ix) = model(iz+2,ix) + c4*data(iz,ix)*aux
          model(iz+1,ix) = model(iz+1,ix) + c5*data(iz,ix)*aux
          model(iz  ,ix) = model(iz  ,ix) - c5*data(iz,ix)*aux
          model(iz-1,ix) = model(iz-1,ix) - c4*data(iz,ix)*aux
          model(iz-2,ix) = model(iz-2,ix) - c3*data(iz,ix)*aux
          model(iz-3,ix) = model(iz-3,ix) - c2*data(iz,ix)*aux
          model(iz-4,ix) = model(iz-4,ix) - c1*data(iz,ix)*aux
        enddo
      enddo
    else
      if(.not.add) data = 0.
      do ix=5,nx-5
        do iz=5,nz-5
          data(iz,ix) = data(iz,ix) + (c1*(model(iz+5,ix) - model(iz-4,ix))+&
                                       c2*(model(iz+4,ix) - model(iz-3,ix))+&
                                       c3*(model(iz+3,ix) - model(iz-2,ix))+&
                                       c4*(model(iz+2,ix) - model(iz-1,ix))+&
                                       c5*(model(iz+1,ix) - model(iz  ,ix)))*aux
        enddo
      enddo
    endif
  endsubroutine

  subroutine vxdz(adj,add,model,data)
    real                                  :: aux
    logical,intent(in)                    :: adj,add
    real,dimension(nz,nx),intent(inout) :: model,data

    aux=1.0/dx

    if(adj) then
      if(.not.add) model = 0.
      do ix=6,nx-5
        do iz=6,nz-5
          model(iz+4,ix) = model(iz+4,ix) + c1*data(iz,ix)*aux
          model(iz+3,ix) = model(iz+3,ix) + c2*data(iz,ix)*aux
          model(iz+2,ix) = model(iz+2,ix) + c3*data(iz,ix)*aux
          model(iz+1,ix) = model(iz+1,ix) + c4*data(iz,ix)*aux
          model(iz  ,ix) = model(iz  ,ix) + c5*data(iz,ix)*aux
          model(iz-1,ix) = model(iz-1,ix) - c5*data(iz,ix)*aux
          model(iz-2,ix) = model(iz-2,ix) - c4*data(iz,ix)*aux
          model(iz-3,ix) = model(iz-3,ix) - c3*data(iz,ix)*aux
          model(iz-4,ix) = model(iz-4,ix) - c2*data(iz,ix)*aux
          model(iz-5,ix) = model(iz-5,ix) - c1*data(iz,ix)*aux
        enddo
      enddo
    else
      if(.not.add) data = 0.
      do ix=6,nx-5
        do iz=6,nz-5
          data(iz,ix) = data(iz,ix) + (c1*(model(iz+4,ix) - model(iz-5,ix))+&
                                       c2*(model(iz+3,ix) - model(iz-4,ix))+&
                                       c3*(model(iz+2,ix) - model(iz-3,ix))+&
                                       c4*(model(iz+1,ix) - model(iz-2,ix))+&
                                       c5*(model(iz  ,ix) - model(iz-1,ix)))*aux
        enddo
      enddo
    endif
  endsubroutine

  subroutine vzdx(adj,add,model,data)
    real                                  :: aux
    logical,intent(in)                    :: adj,add
    real,dimension(nz,nx),intent(inout) :: model,data

    aux=1.0/dx

    if(adj)then
      if(.not.add) model = 0.
      do ix=6,nx-5
        do iz=6,nz-5
          model(iz,ix+4) = model(iz,ix+4) + c1*data(iz,ix)*aux
          model(iz,ix+3) = model(iz,ix+3) + c2*data(iz,ix)*aux
          model(iz,ix+2) = model(iz,ix+2) + c3*data(iz,ix)*aux
          model(iz,ix+1) = model(iz,ix+1) + c4*data(iz,ix)*aux
          model(iz,ix  ) = model(iz,ix  ) + c5*data(iz,ix)*aux
          model(iz,ix-1) = model(iz,ix-1) - c5*data(iz,ix)*aux
          model(iz,ix-2) = model(iz,ix-2) - c4*data(iz,ix)*aux
          model(iz,ix-3) = model(iz,ix-3) - c3*data(iz,ix)*aux
          model(iz,ix-4) = model(iz,ix-4) - c2*data(iz,ix)*aux
          model(iz,ix-5) = model(iz,ix-5) - c1*data(iz,ix)*aux
      enddo
    enddo
    else
      if(.not.add) data = 0.
        do ix=6,nx-5
          do iz=6,nz-5
          data(iz,ix) = data(iz,ix) + (c1*(model(iz,ix+4) - model(iz,ix-5))+&
                                       c2*(model(iz,ix+3) - model(iz,ix-4))+&
                                       c3*(model(iz,ix+2) - model(iz,ix-3))+&
                                       c4*(model(iz,ix+1) - model(iz,ix-2))+&
                                       c5*(model(iz,ix  ) - model(iz,ix-1)))*aux
        enddo
      enddo
    endif
  endsubroutine

endmodule

