module inplace_mod
  use model_type_mod
  use sep
  use adj_mod
  use dottest
  use solver_reg_mod
  use cgstep_mod
  implicit none
  real,pointer :: x(:),y(:)
  
  contains
  
  
  subroutine inplace_it(in,out,icom)
    type(vel_model) :: in,out
    integer :: icom,rngT(2),rngB(2),i3,ierr
    character(len=128) :: sepfile,v1
    logical :: up_hills,down_decrease,implace
    real :: down_dist,down_amount
    real, allocatable :: model(:,:,:)
    integer, allocatable :: bot(:,:),top(:,:)

    
    call create_new_model(in,out,0,0)
    call cat_it(v1,"file_",icom);   call from_param(v1,sepfile)
    call cat_it(v1,"emplace_",icom); call from_param(v1,implace)
    call cat_it(v1,"conform_",icom); call from_param(v1,up_hills)
    call cat_it(v1,"down_decrease_",icom); call from_param(v1,down_decrease)
        call cat_it(v1,"down_amount_",icom); call from_param(v1,down_amount)

    call cat_it(v1,"down_dist_",icom); call from_param(v1,down_dist,0.)
    allocate(model(in%n1,in%n2,in%n3))
    if(.not. check_load(trim(sepfile),in,model)) then
      call seperr("model load problem")
    end if
  
    allocate(top(in%n2,in%n3))
    allocate(bot(in%n2,in%n3))   
    

  
    
    call find_top(model,top,rngT)
    call find_bot(model,bot,rngB)
        write(0,*) "FINISHED 3"



    if(up_hills) then
      call apply_hills(model,top,bot,in,out,rngT)
    end if
    write(0,*) "FINISHED HILLS"
    if(implace)   call add_model(out,model,out,icom)
    write(0,*) "FINISHE ADD"
    if(down_decrease) then
      call apply_gauss_down(bot,down_amount,down_dist,out)
    end if
    write(0,*) "FINISHED GAUSS"
    deallocate(model,top,bot) 
end subroutine


subroutine smooth_3d(nlen,fld,ntms)
  real :: fld(:,:,:)
  integer   :: nlen(3),ntms,itm
  real,allocatable,dimension(:,:,:) :: t1,t2
  real :: bsum
  integer :: i1,i2,i3,nsz(3)

  nsz=(/size(fld,1),size(fld,2),size(fld,3)/)
  allocate(t1(nsz(1),nsz(2),nsz(3)),t2(nsz(1),nsz(2),nsz(3)))
  t1=fld
  t2=0
  
   do itm=1,ntms!{
  !$OMP PARALLEL DO PRIVATE(i2,i3,i1,bsum)
    do i3=1,nsz(3)!{
     
      !AXIS 2
      do i2=1,nsz(2)!{
        bsum=sum(t1(1:nlen(1)*2+1,i2,i3))
        t2(1:nlen(1)+1,i2,i3)=bsum/(nlen(1)*2+1)
        do i1=nlen(1)+2,nsz(1)-nlen(1)-1!{
          bsum=bsum-t1(i1-nlen(1)-1,i2,i3)+t1(i1+nlen(1),i2,i3)
          t2(i1,i2,i3)=bsum/(nlen(1)*2+1)
        end do!}
        t2(nsz(1)-nlen(1):,i2,i3)=t1(nsz(1)-nlen(1)-1,i2,i3)
        bsum=sum(t2(nsz(1)-nlen(1)*2:,i2,i3))
       
        t1(nsz(1)-nlen(1),i2,i3)=bsum/(nlen(1)*2+1)
        do i1=nsz(1)-nlen(1)-1,nlen(1)+1,-1!{
          bsum=bsum-t2(i1+nlen(1)+1,i2,i3)+t2(i1-nlen(1),i2,i3)
          
          t1(i1,i2,i3)=bsum/(nlen(1)*2+1)
        end do!}
        t1(1:nlen(1),i2,i3)=t2(1:nlen(1),i2,i3)
        
     end do!}
     

  !   write(0,*) "AXIS 1 done",i3
     do i1=1,nsz(1)!{
       bsum=sum(t1(i1,1:nlen(2)*2+1,i3))
         
       t2(i1,1:nlen(2)+1,i3)=bsum/(nlen(2)*2+1)
       do i2=nlen(2)+2,nsz(2)-nlen(2)-1!{
       
         bsum=bsum-t1(i1,i2-nlen(2)-1,i3)+t1(i1,i2+nlen(2),i3)
         
         t2(i1,i2,i3)=bsum/(nlen(2)*2+1)
       end do!}
      
       t2(i1,nsz(2)-nlen(2):,i3)=t1(i1,nsz(2)-nlen(2),i3)
       bsum=sum(t2(i1,nsz(2)-nlen(2)*2:,i3))
       t1(i1,nsz(2)-nlen(2),i3)=bsum/(nlen(2)*2+1)

      

       do i2=nsz(2)-nlen(2)-1,nlen(2)+1,-1 !{
         bsum=bsum-t2(i1,i2+nlen(2)+1,i3)+t2(i1,i2-nlen(2),i3)
         t1(i1,i2,i3)=bsum/(nlen(2)*2+1)
       end do !}
       t1(i1,1:nlen(2),i3)=t2(i1,1:nlen(2),i3)
  
     end do !}
     !    write(0,*) "AXIS 2 done",i3
   end do!}
 !$OMP END PARALLEL DO

   !AXIS 1 and 2 done
 !$OMP PARALLEL DO PRIVATE(i2,i3,i1,bsum)
    do i2=1,nsz(2)
      !AXIS 1
      do i1=1,nsz(1)
        bsum=sum(t1(i1,i2,1:nlen(3)*2+1))

        t2(i1,i2,1:nlen(3)+1)=bsum/(nlen(3)*2+1)
        do i3=nlen(3)+2,nsz(3)-nlen(3)-1
          bsum=bsum-t1(i1,i2,i3-nlen(3)-1)+t1(i1,i2,i3+nlen(3))
          t2(i1,i2,i3)=bsum/(nlen(3)*2+1)

        end do
        t2(i1,i2,nsz(3)-nlen(3):)=bsum/(nlen(3)*2+1)
    

        bsum=sum(t2(i1,i2,nsz(3)-nlen(3)*2:))
        
     
        t1(i1,i2,nsz(3)-nlen(3):)=bsum/(nlen(3)*2+1)
        do i3=nsz(3)-nlen(3)-1,nlen(3)+1,-1
          bsum=bsum-t2(i1,i2,i3+nlen(3)+1)+t2(i1,i2,i3-nlen(3))
                
       
          if(bsum/(nlen(3)*2+1)>2.) then
        end if
          t1(i1,i2,i3)=bsum/(nlen(3)*2+1)
        end do
        t1(i1,i2,1:nlen(3))=t2(i1,i2,1:nlen(3))
 
     end do
   end do
  !$OMP END PARALLEL DO


  end do
  fld=t1
  deallocate(t1,t2)
end subroutine


subroutine apply_gauss_down(surf,down_decrease,down_dist,out)
  type(vel_model) :: out
  integer :: surf(:,:)
  real    :: down_decrease,down_dist
  real, allocatable :: t1(:,:,:),t2(:,:,:),jnk(:,:)
  integer :: i1,i2,i3,ib,itm,nlen
  integer :: imin,nsz(3),idist
  real   :: bsum
  imin=minval(surf)
  nsz=(/out%n1-imin+6,out%n2,out%n3/)
  allocate(t1(nsz(1),nsz(2),nsz(3)))
  allocate(jnk(nsz(2),nsz(3)))
  call random_number(jnk)
  idist=down_dist/out%d1-12

  jnk=(1+(Jnk-.5)/5.)*down_decrease
  t1=0; t2=0;
  nlen=down_dist/out%d1
  !$OMP PARALLEL DO private(i2,i3)
  do i3=1,out%n3
    do i2=1,out%n2
      if(surf(i2,i3) < out%n1) then
        t1(max(1,surf(i2,i3)-imin+1):min(surf(i2,i3)-imin+nlen+1+idist,nsz(1)),i2,i3)=jnk(i2,i3)
      end if
    end do
  end do
  call smooth_3d((/6,6,6/),t1,3)
 !$OMP PARALLEL DO private(i1,i2,i3)
   do i3=1,out%n3
     do i2=1,out%n2
       if(surf(i2,i3) < out%n1) then
    !     write(0,*) "SIZE",surf(i2,i3),size(out%vals,1),surf(i2,i3)-imin+nlen+1,size(t1,1)
      out%vals(surf(i2,i3):,i2,i3)=out%vals(surf(i2,i3):,i2,i3)-t1(surf(i2,i3)-imin+6:,i2,i3)
     !             out%vals(surf(i2,i3):,i2,i3)=t1(surf(i2,i3)-imin+6:,i2,i3)

       end if
    end do
  end do
 !$OMP END PARALLEL DO
  deallocate(t1,jnk)
end subroutine
  

subroutine fill_it(top,closest,ax,ndist,i2,i3)
  integer :: top(:,:)
  integer :: closest(:,:,:)
  integer :: ax(2),ndist,i2,i3,iloc2,iloc3
  integer :: i
  logical :: found
  
  found=.false. ;i=1
  iloc2=i2; iloc3=i3
  do while (.not. found .and. i < ndist) 
     iloc2=iloc2+ax(1)
     iloc3=iloc3+ax(2)
     if(iloc2<1 .or. iloc3 < 1 .or. iloc2 > size(top,1) .or. iloc3 > size(top,2)) then
       found=.true.
     else
       if(top(iloc2,iloc3) >0) then
         found=.true.
       else
         if(closest(1,iloc2,iloc3)<0) then
           closest(:,iloc2,iloc3)=closest(:,i2,i3)
          else
           if((closest(1,iloc2,iloc3)-iloc2)**2+(closest(2,iloc2,iloc3)-iloc3)**2 >&
            (iloc2-closest(1,i2,i3))**2+(iloc3-closest(2,i2,i3))**2) then
             closest(:,iloc2,iloc3)=closest(:,i2,i3)
           else
             found=.true.
           end if
         end if
       end if
     end if
   end do
 end subroutine





subroutine apply_hills(model,top,bot,in,out,rngT)
  type(vel_model) :: out,in
  integer :: bot(:,:),top(:,:)
  real,allocatable :: up(:,:,:)
  real :: model(:,:,:)
  real    :: f,dist
  integer :: ibeg,iend,rngT(2)
  real :: dmax,pi
  real  :: distSalt,shift
  integer :: i1,i2,i3,iuse,ia,b2,e2,b3,e3,ib
  integer :: die,itop,imaxSearch,isearch ,useT,useB
  integer :: upMaxShift,i0,iold,irng(2)
  logical :: found
  
  allocate(up(out%n1,out%n2,out%n3))
  up=model/4490.
  call smooth_3d((/30,15,15/),up,3)
  do i3=1,out%n3
    do i2=1,out%n2
      do i1=1,out%n1
        if(bot(i2,i3)< out%n1 .and. i1 > bot(i2,i3)) up(i1,i2,i3)=up(i1,i2,i3)*.05
      end do
    end do
  end do
  call smooth_3d((/30,15,15/),up,4)
  do i3=1,out%n3
    do i2=1,out%n2
      do i1=1,out%n1
        ia=up(i1,i2,i3)/3.*500.+i1+.5
        if(ia>out%n1) ia=out%n1
            out%vals(i1,i2,i3)=in%vals(ia,i2,i3)
            out%layer(i1,i2,i3)=in%layer(ia,i2,i3)
          end do!}
      end do!}
    end do!}
  deallocate(up)
end subroutine
  

subroutine find_bot(model,surf,rng)
 real :: model(:,:,:)
 integer :: surf(:,:)
 integer :: i1,i2,i3,rng(2)
 logical :: found


 rng(1)=size(model,1)
 rng(2)=1
 do i3=1,size(model,3)
   do i2=1,size(model,2)
     found=.false.; i1=size(model,1)
     surf(i2,i3)=size(model,1)+10
     do while (.not. found .and. i1 >1)
       if(model(i1,i2,i3) > .1) then
         surf(i2,i3)=i1
         rng(1)=min(rng(1),i1)
         rng(2)=max(rng(2),i1)
         found=.true.
       else
         i1=i1-1
       end if
      end do
    end do
  end do

end subroutine
subroutine find_top(model,surf,rng)
 real :: model(:,:,:)
 integer :: surf(:,:)
 integer :: i1,i2,i3,rng(2)
 logical :: found
 
 
 rng(1)=size(model,1)
 rng(2)=1
!!! !$OMP PARALLEL DO private(i1,i2,i3,found)
 do i3=1,size(model,3)
   do i2=1,size(model,2)
     found=.false.; i1=1
     surf(i2,i3)=-1
     do while (.not. found .and. i1 < size(model,1))
       if(model(i1,i2,i3) > .1) then
         surf(i2,i3)=i1
         rng(1)=min(rng(1),i1)
         rng(2)=max(rng(2),i1)
         found=.true.
       else
         i1=i1+1
       end if
      end do
    end do
  end do


end subroutine

subroutine add_model(in,model,out,isurf)
  integer :: isurf
  type(vel_model) :: in,out
  real :: model(:,:,:)
  integer :: i3,i2,i1
  
  !$OMP PARALLEL DO private(i1,i2,i3)
  do i3=1,out%n3
    do i2=1,out%n2
      do i1=1,out%n1
        if(model(i1,i2,i3) > .1) then
          out%layer(i1,i2,i3)=isurf
          out%vals(i1,i2,i3)=model(i1,i2,i3)
        end if
      end do
    end do
  end do
  
  
end subroutine

logical function check_load(sepfile,in,model)
 type(vel_model) :: in
 character(len=*) :: sepfile
 real :: model(:,:,:)
 real, allocatable :: tt(:,:)
 integer :: n(3),i,n2(3),ierr
 real :: o(3),d(3),o2(3),d2(3)
 character(len=128) :: label
 
 n2=(/in%n1,in%n2,in%n3/)
 o2=(/in%o1,in%o2,in%o3/)
 d2=(/in%d1,in%d2,in%d3/)
 check_load=.false.
 model=0
 do i=1,3
   !ierr= sep_get_data_axis_par(sepfile,i,n(i),o(i),d(i),label)
   call sep_get_data_axis_par(sepfile,i,n(i),o(i),d(i),label)
   if((i==1 .and. n2(i)<n(i)) .or. (i>1 .and. n2(i)/=n(i))) then
     write(0,*) "AXIS ",i,"ns do not match ",n(i),n2(i)
     return
   end if
   if(abs((d2(i)-d(i))/d2(i)) >.001) then
     write(0,*) "AXIS",i,"ds do not match",d(i),d2(i)
     return
   end if
   if(abs((o2(i)-o(i))/o2(i)) > .001) then
      write(0,*) "AXIS",i,"os do not match",o(i),o2(i)
      return
   end if
end do
allocate(tt(n(1),n(2)))
do i=1,n(3)
  !if(n(1)*n(2)*4/=sreed(sepfile,tt,size(tt)*4)) then
  !   write(0,*) "trouble reading file"
  !   return
  !end if
  call sreed(sepfile,tt,size(tt)*4)
  model(1:n(1),:,i)=tt
end do
check_load=.true.
deallocate(tt)
end function


end module
