! Linear interpolation operator module
module lint_mod

  implicit none
  real, private                         :: dt, dtseis, aux
  integer, private                      :: nrec, ntseis, nt, nz, nx

  contains

  subroutine lint_init(dt_in,dtseis_in,nrec_in,nz_in,nx_in,ntseis_in,nt_in)
    integer                     :: nrec_in, ntseis_in, nt_in, nz_in, nx_in
    real                        :: dt_in, dtseis_in

    nrec = nrec_in
    ntseis = ntseis_in
    dt = dt_in
    nt = nt_in
    dtseis = dtseis_in
    nz = nz_in
    nx = nx_in

    aux = sqrt(float(ntseis)/float(nt))

  end subroutine

!==================================
! FUNCTION CALLS
!==================================
  function lint2d_op(adj,add,model,data,it) result(stat)
    logical,intent(in)          :: adj,add
    real,dimension(:)         :: model,data
    integer                     :: stat,it

    call lint2d_op2(adj,add,model,data,it)
    stat=0
  endfunction

  function lint3d_op(adj,add,model,data,it) result(stat)
    logical,intent(in)          :: adj,add
    real,dimension(:)         :: model,data
    integer                     :: stat,it

    call lint3d_op2(adj,add,model,data,it)
    stat=0
  endfunction
!==================================
  subroutine lint2d_op2(adj, add, model, data, it)
    logical,intent(in)            :: adj, add
    integer                       :: it, ix, itseis
    real, dimension(nrec)       :: data
    real, dimension(nrec,ntseis):: model
    real                        :: temp, f, g

    if (adj) then
      temp = it*dt/dtseis
      itseis = temp
      f = temp - itseis
      g = 1.0 - f
      if (add==0) then
        model(:,itseis+1)=0.
        model(:,itseis+2)=0.
      endif
      model(:,itseis+1) =  model(:,itseis+1) + aux*g*data(:) 
      model(:,itseis+2) =  model(:,itseis+2) + aux*f*data(:) 
    else
      if (add==0) data(:)=0.
        temp = (it*dt)/dtseis
        itseis = temp
        f = temp-itseis
        g = 1.0 - f
        data(:) = data(:) + aux*(g*model(:,itseis+1) + f*model(:,itseis+2))
    endif

    !write(0,*)"itseis=",itseis+2

  endsubroutine

  subroutine lint3d_op2(adj, add, model, data, it)
    logical,intent(in)             :: adj, add
    integer                        :: it, ix, itseis
    real, dimension(nz,nx)       :: data
    real, dimension(nz,nx,ntseis):: model
    real                         :: temp, f, g

    if (adj) then
      temp = it*dt/dtseis
      itseis = temp
      f = temp - itseis
      g = 1.0 - f
      if (add==0) then
        model(:,:,itseis+1)=0.
        model(:,:,itseis+2)=0.
      endif
        model(:,:,itseis+1) =  model(:,:,itseis+1) + aux*g*data(:,:) 
        model(:,:,itseis+2) =  model(:,:,itseis+2) + aux*f*data(:,:) 
    else
      if (add==0) data(:,:)=0.
        temp = (it*dt)/dtseis
        itseis = temp
        f = temp-itseis
        g = 1.0 - f
        data(:,:) = aux*(g*model(:,:,itseis+1) + f*model(:,:,itseis+2))
    endif

  endsubroutine

  subroutine lint3d_op3(adj, add, model1,model2, data, it)
    logical,intent(in)             :: adj, add
    integer                        :: it, ix, itseis
    real, dimension(nz,nx)       :: data
    real, dimension(nz,nx)       :: model1,model2
    real                         :: temp, f, g

    if (adj) then
      temp = it*dt/dtseis
      itseis = temp
      f = temp - itseis
      g = 1.0 - f
      if (add==0) then
        model1(:,:)=0.
        model2(:,:)=0.
      endif
        model1(:,:) =  model1(:,:) + aux*g*data(:,:) 
        model2(:,:) =  model2(:,:) + aux*f*data(:,:) 
    else
      if (add==0) data(:,:)=0.
        temp = (it*dt)/dtseis
        itseis = temp
        f = temp-itseis
        g = 1.0 - f
        data(:,:) = aux*(g*model1(:,:) + f*model2(:,:))
    endif

  endsubroutine


end module
