! Program created by Gustavo Catao Alves at
! the Stanford Exploration Project
! May, 6th, 2014

! seismogram module
module seis_mod_rtm

  use sep

  implicit none
  integer, private :: nx,nt,nz
  real, private    :: dx,dt,dz
  real, dimension(:,:), pointer, private   :: Vx_rec,Vz_rec
  real, dimension(:,:), pointer, private   :: sigmaxx_rec,sigmazz_rec,sigmaxz_rec
  real, dimension(:,:), pointer, private   :: sigmaxx,sigmazz,sigmaxz
  real, dimension(:,:), pointer, private   :: vx0new,vx0cur,vz0new,vz0cur
  real, dimension(:,:), pointer, private   :: sigmaxx0new,sigmaxx0cur,sigmazz0new,sigmazz0cur,sigmaxz0new,sigmaxz0cur
  real, dimension(:,:), pointer, private   :: dm1,dm2,dm3
  real, dimension(:,:,:),allocatable,private :: temp_array
  logical,private                          :: lame

  contains

  subroutine seismo(nx_in,dx_in,nt_in,dt_in,Vx_rec_in,Vz_rec_in,sigmaxx_rec_in,sigmazz_rec_in,sigmaxz_rec_in)
    integer,intent(in)           :: nx_in,nt_in
    real,intent(in)              :: dx_in,dt_in
    real, dimension(:,:), target :: Vx_rec_in,Vz_rec_in,sigmaxx_rec_in,sigmazz_rec_in,sigmaxz_rec_in

    nx = nx_in
    dx = dx_in
    nt = nt_in
    dt = dt_in
    Vx_rec => Vx_rec_in
    Vz_rec => Vz_rec_in
    sigmaxx_rec => sigmaxx_rec_in
    sigmazz_rec => sigmazz_rec_in
    sigmaxz_rec => sigmaxz_rec_in

    call to_history("n1", nx, "Vx_rec")
    call to_history("o1", 0., "Vx_rec")
    call to_history("d1", dx, "Vx_rec")
    call to_history("n2", nt, "Vx_rec")
    call to_history("o2", 0., "Vx_rec")
    call to_history("d2", dt, "Vx_rec")
         
    call sep_write(Vx_rec, "Vx_rec")

    call to_history("n1", nx, "Vz_rec")
    call to_history("o1", 0., "Vz_rec")
    call to_history("d1", dx, "Vz_rec")
    call to_history("n2", nt, "Vz_rec")
    call to_history("o2", 0., "Vz_rec")
    call to_history("d2", dt, "Vz_rec")
         
    call sep_write(Vz_rec, "Vz_rec")

    call to_history("n1", nx, "sigmaxx_rec")
    call to_history("o1", 0., "sigmaxx_rec")
    call to_history("d1", dx, "sigmaxx_rec")
    call to_history("n2", nt, "sigmaxx_rec")
    call to_history("o2", 0., "sigmaxx_rec")
    call to_history("d2", dt, "sigmaxx_rec")
         
    call sep_write(sigmaxx_rec, "sigmaxx_rec")

    call to_history("n1", nx, "sigmazz_rec")
    call to_history("o1", 0., "sigmazz_rec")
    call to_history("d1", dx, "sigmazz_rec")
    call to_history("n2", nt, "sigmazz_rec")
    call to_history("o2", 0., "sigmazz_rec")
    call to_history("d2", dt, "sigmazz_rec")
         
    call sep_write(sigmazz_rec, "sigmazz_rec")

    call to_history("n1", nx, "sigmaxz_rec")
    call to_history("o1", 0., "sigmaxz_rec")
    call to_history("d1", dx, "sigmaxz_rec")
    call to_history("n2", nt, "sigmaxz_rec")
    call to_history("o2", 0., "sigmaxz_rec")
    call to_history("d2", dt, "sigmaxz_rec")
        
    call sep_write(sigmaxz_rec, "sigmaxz_rec")

  endsubroutine

  subroutine seismo2(nz_in,dz_in,nx_in,dx_in,dt_in,vx0new_in,vx0cur_in,vz0new_in,vz0cur_in,sigmaxx0new_in,sigmaxx0cur_in,sigmazz0new_in,sigmazz0cur_in,sigmaxz0new_in,sigmaxz0cur_in)
    integer,intent(in)            :: nz_in,nx_in
    real,intent(in)               :: dz_in,dx_in,dt_in
    real, dimension(:,:),target :: vx0new_in,vx0cur_in,vz0new_in,vz0cur_in
    real, dimension(:,:),target :: sigmaxx0new_in,sigmaxx0cur_in,sigmazz0new_in,sigmazz0cur_in,sigmaxz0new_in,sigmaxz0cur_in

    nz = nz_in
    dz = dz_in
    nx = nx_in
    dx = dx_in
    nt = 2
    dt = dt_in
    vx0new => vx0new_in
    vx0cur => vx0cur_in
    vz0new => vz0new_in
    vz0cur => vz0cur_in
    sigmaxx0new => sigmaxx0new_in
    sigmaxx0cur => sigmaxx0cur_in
    sigmazz0new => sigmazz0new_in
    sigmazz0cur => sigmazz0cur_in
    sigmaxz0new => sigmaxz0new_in
    sigmaxz0cur => sigmaxz0cur_in
    
    allocate(temp_array(nz,nx,2))

    call to_history("n1", nz, "vx0")
    call to_history("o1", 0., "vx0")
    call to_history("d1", dz, "vx0")
    call to_history("n2", nx, "vx0")
    call to_history("o2", 0., "vx0")
    call to_history("d2", dx, "vx0")
    call to_history("n3", nt, "vx0")
    call to_history("o3", 0., "vx0")
    call to_history("d3", dt, "vx0")
 
    temp_array(:,:,1) = vx0cur(:,:)
    temp_array(:,:,2) = vx0new(:,:)
         
    call sep_write(temp_array, "vx0")

    call to_history("n1", nz, "vz0")
    call to_history("o1", 0., "vz0")
    call to_history("d1", dz, "vz0")
    call to_history("n2", nx, "vz0")
    call to_history("o2", 0., "vz0")
    call to_history("d2", dx, "vz0")
    call to_history("n3", nt, "vz0")
    call to_history("o3", 0., "vz0")
    call to_history("d3", dt, "vz0")
 
    temp_array(:,:,1) = vz0cur(:,:)
    temp_array(:,:,2) = vz0new(:,:)
         
    call sep_write(temp_array, "vz0")

    call to_history("n1", nz, "sigmaxx0")
    call to_history("o1", 0., "sigmaxx0")
    call to_history("d1", dz, "sigmaxx0")
    call to_history("n2", nx, "sigmaxx0")
    call to_history("o2", 0., "sigmaxx0")
    call to_history("d2", dx, "sigmaxx0")
    call to_history("n3", nt, "sigmaxx0")
    call to_history("o3", 0., "sigmaxx0")
    call to_history("d3", dt, "sigmaxx0")
    
    temp_array(:,:,1) = sigmaxx0cur(:,:)
    temp_array(:,:,2) = sigmaxx0new(:,:)    
         
    call sep_write(temp_array, "sigmaxx0")

    call to_history("n1", nz, "sigmazz0")
    call to_history("o1", 0., "sigmazz0")
    call to_history("d1", dz, "sigmazz0")
    call to_history("n2", nx, "sigmazz0")
    call to_history("o2", 0., "sigmazz0")
    call to_history("d2", dx, "sigmazz0")
    call to_history("n3", nt, "sigmazz0")
    call to_history("o3", 0., "sigmazz0")
    call to_history("d3", dt, "sigmazz0")
         
    temp_array(:,:,1) = sigmazz0cur(:,:)
    temp_array(:,:,2) = sigmazz0new(:,:)    
         
    call sep_write(temp_array, "sigmazz0")
    
    call to_history("n1", nz, "sigmaxz0")
    call to_history("o1", 0., "sigmaxz0")
    call to_history("d1", dz, "sigmaxz0")
    call to_history("n2", nx, "sigmaxz0")
    call to_history("o2", 0., "sigmaxz0")
    call to_history("d2", dx, "sigmaxz0")
    call to_history("n3", nt, "sigmaxz0")
    call to_history("o3", 0., "sigmaxz0")
    call to_history("d3", dt, "sigmaxz0")
         
    temp_array(:,:,1) = sigmaxz0cur(:,:)
    temp_array(:,:,2) = sigmaxz0new(:,:)    
         
    call sep_write(temp_array, "sigmaxz0")
    
    deallocate(temp_array)

  endsubroutine

  subroutine img(nx_in,dx_in,nz_in,dz_in,dm1_in,dm2_in,dm3_in,lame_in)
    integer,intent(in)           :: nx_in,nz_in
    real,intent(in)              :: dx_in,dz_in
    real, dimension(:,:), target :: dm1_in,dm2_in,dm3_in
    logical                      :: lame_in

    nx = nx_in
    dx = dx_in
    nz = nz_in
    dz = dz_in
    dm1 => dm1_in
    dm2 => dm2_in
    dm3 => dm3_in
    lame = lame_in

    call to_history("n1", nz, "drho")
    call to_history("o1", 0., "drho")
    call to_history("d1", dz, "drho")
    call to_history("n2", nx, "drho")
    call to_history("o2", 0., "drho")
    call to_history("d2", dx, "drho")
         
    call sep_write(dm1, "drho")

    if (lame) then
      call to_history("n1", nz, "dlambda")
      call to_history("o1", 0., "dlambda")
      call to_history("d1", dz, "dlambda")
      call to_history("n2", nx, "dlambda")
      call to_history("o2", 0., "dlambda")
      call to_history("d2", dx, "dlambda")
      call sep_write(dm2, "dlambda")

      call to_history("n1", nz, "dmu")
      call to_history("o1", 0., "dmu")
      call to_history("d1", dz, "dmu")
      call to_history("n2", nx, "dmu")
      call to_history("o2", 0., "dmu")
      call to_history("d2", dx, "dmu")    
      call sep_write(dm3, "dmu")
    else
      call to_history("n1", nz, "dvp")
      call to_history("o1", 0., "dvp")
      call to_history("d1", dz, "dvp")
      call to_history("n2", nx, "dvp")
      call to_history("o2", 0., "dvp")
      call to_history("d2", dx, "dvp")
      call sep_write(dm2, "dvp")
         
      call to_history("n1", nz, "dvs")
      call to_history("o1", 0., "dvs")
      call to_history("d1", dz, "dvs")
      call to_history("n2", nx, "dvs")
      call to_history("o2", 0., "dvs")
      call to_history("d2", dx, "dvs")    
      call sep_write(dm3, "dvs")
    endif


  endsubroutine

endmodule
