! Program created by Gustavo Catao Alves at
! the Stanford Exploration Project
! May, 20th, 2015

! seismogram module
module snap_mod_rtm

  use sep

  implicit none
  integer, private :: nz,nx,nt,nt2
  real, private    :: dz,dx,dt,dt2
  real, dimension(:,:), allocatable, private :: temparray

  integer, private :: it,countsnap,nsnap,dsnap

  contains

  ! Initialization of all variables needed by this subroutine
  subroutine snap_init(nz_in,dz_in,nx_in,dx_in,nt_in,dt_in)
    integer,intent(in)           :: nz_in,nx_in,nt_in
    real,intent(in)              :: dz_in,dx_in,dt_in

    nz = nz_in
    dz = dz_in
    nx = nx_in
    dx = dx_in
    nt = nt_in
    dt = dt_in

    allocate(temparray(nz,nx))
    temparray = 0.

 	call to_history("n1", nz, "dat/vx_bkg.H")
    call to_history("o1", 0., "dat/vx_bkg.H")
    call to_history("d1", dz, "dat/vx_bkg.H")
    call to_history("n2", nx, "dat/vx_bkg.H")
    call to_history("o2", 0., "dat/vx_bkg.H")
    call to_history("d2", dx, "dat/vx_bkg.H")
    call to_history("n3", nt, "dat/vx_bkg.H")
    call to_history("o3", 0., "dat/vx_bkg.H")
    call to_history("d3", dt, "dat/vx_bkg.H")
    call to_history("n1", nz, "dat/vz_bkg.H")
    call to_history("o1", 0., "dat/vz_bkg.H")
    call to_history("d1", dz, "dat/vz_bkg.H")
    call to_history("n2", nx, "dat/vz_bkg.H")
    call to_history("o2", 0., "dat/vz_bkg.H")
    call to_history("d2", dx, "dat/vz_bkg.H")
    call to_history("n3", nt, "dat/vz_bkg.H")
    call to_history("o3", 0., "dat/vz_bkg.H")
    call to_history("d3", dt, "dat/vz_bkg.H") 
    call to_history("n1", nz, "dat/sigmaxx_bkg.H")
    call to_history("o1", 0., "dat/sigmaxx_bkg.H")
    call to_history("d1", dz, "dat/sigmaxx_bkg.H")
    call to_history("n2", nx, "dat/sigmaxx_bkg.H")
    call to_history("o2", 0., "dat/sigmaxx_bkg.H")
    call to_history("d2", dx, "dat/sigmaxx_bkg.H")
    call to_history("n3", nt, "dat/sigmaxx_bkg.H")
    call to_history("o3", 0., "dat/sigmaxx_bkg.H")
    call to_history("d3", dt, "dat/sigmaxx_bkg.H")
    call to_history("n1", nz, "dat/sigmazz_bkg.H")
    call to_history("o1", 0., "dat/sigmazz_bkg.H")
    call to_history("d1", dz, "dat/sigmazz_bkg.H")
    call to_history("n2", nx, "dat/sigmazz_bkg.H")
    call to_history("o2", 0., "dat/sigmazz_bkg.H")
    call to_history("d2", dx, "dat/sigmazz_bkg.H")
    call to_history("n3", nt, "dat/sigmazz_bkg.H")
    call to_history("o3", 0., "dat/sigmazz_bkg.H")
    call to_history("d3", dt, "dat/sigmazz_bkg.H")
    call to_history("n1", nz, "dat/sigmaxz_bkg.H")
    call to_history("o1", 0., "dat/sigmaxz_bkg.H")
    call to_history("d1", dz, "dat/sigmaxz_bkg.H")
    call to_history("n2", nx, "dat/sigmaxz_bkg.H")
    call to_history("o2", 0., "dat/sigmaxz_bkg.H")
    call to_history("d2", dx, "dat/sigmaxz_bkg.H")
    call to_history("n3", nt, "dat/sigmaxz_bkg.H")
    call to_history("o3", 0., "dat/sigmaxz_bkg.H")
    call to_history("d3", dt, "dat/sigmaxz_bkg.H")
 
 	call to_history("n1", nz, "dat/vx_rtm.H")
    call to_history("o1", 0., "dat/vx_rtm.H")
    call to_history("d1", dz, "dat/vx_rtm.H")
    call to_history("n2", nx, "dat/vx_rtm.H")
    call to_history("o2", 0., "dat/vx_rtm.H")
    call to_history("d2", dx, "dat/vx_rtm.H")
    call to_history("n3", nt, "dat/vx_rtm.H")
    call to_history("o3", 0., "dat/vx_rtm.H")
    call to_history("d3", dt, "dat/vx_rtm.H")
    call to_history("n1", nz, "dat/vz_rtm.H")
    call to_history("o1", 0., "dat/vz_rtm.H")
    call to_history("d1", dz, "dat/vz_rtm.H")
    call to_history("n2", nx, "dat/vz_rtm.H")
    call to_history("o2", 0., "dat/vz_rtm.H")
    call to_history("d2", dx, "dat/vz_rtm.H")
    call to_history("n3", nt, "dat/vz_rtm.H")
    call to_history("o3", 0., "dat/vz_rtm.H")
    call to_history("d3", dt, "dat/vz_rtm.H")   
	call to_history("n1", nz, "dat/sigmaxx_rtm.H")
    call to_history("o1", 0., "dat/sigmaxx_rtm.H")
    call to_history("d1", dz, "dat/sigmaxx_rtm.H")
    call to_history("n2", nx, "dat/sigmaxx_rtm.H")
    call to_history("o2", 0., "dat/sigmaxx_rtm.H")
    call to_history("d2", dx, "dat/sigmaxx_rtm.H")
    call to_history("n3", nt, "dat/sigmaxx_rtm.H")
    call to_history("o3", 0., "dat/sigmaxx_rtm.H")
    call to_history("d3", dt, "dat/sigmaxx_rtm.H")
    call to_history("n1", nz, "dat/sigmazz_rtm.H")
    call to_history("o1", 0., "dat/sigmazz_rtm.H")
    call to_history("d1", dz, "dat/sigmazz_rtm.H")
    call to_history("n2", nx, "dat/sigmazz_rtm.H")
    call to_history("o2", 0., "dat/sigmazz_rtm.H")
    call to_history("d2", dx, "dat/sigmazz_rtm.H")
    call to_history("n3", nt, "dat/sigmazz_rtm.H")
    call to_history("o3", 0., "dat/sigmazz_rtm.H")
    call to_history("d3", dt, "dat/sigmazz_rtm.H")
    call to_history("n1", nz, "dat/sigmaxz_rtm.H")
    call to_history("o1", 0., "dat/sigmaxz_rtm.H")
    call to_history("d1", dz, "dat/sigmaxz_rtm.H")
    call to_history("n2", nx, "dat/sigmaxz_rtm.H")
    call to_history("o2", 0., "dat/sigmaxz_rtm.H")
    call to_history("d2", dx, "dat/sigmaxz_rtm.H")
    call to_history("n3", nt, "dat/sigmaxz_rtm.H")
    call to_history("o3", 0., "dat/sigmaxz_rtm.H")
    call to_history("d3", dt, "dat/sigmaxz_rtm.H")
    
  endsubroutine

  subroutine snap_close()
    deallocate(temparray)
  endsubroutine

!  subroutine snap_op(sigmaxx,S3,dm2)
!    integer                          :: wstat,srite
!    real,dimension(nz,nx),intent(in) :: sigmaxx,S3,dm2

!    temparray(:,1:nx) = sigmaxx(:,:)/(maxval(sigmaxx(:,:))+1)
!    temparray(:,nx+1:2*nx) = temparray(:,nx+1:2*nx) + dm2(:,:)/(maxval(dm2(:,:))+1)
!    temparray(:,2*nx+1:3*nx) = S3(:,:)/(maxval(S3(:,:))+1)

!    wstat = srite(fname,temparray,4*nz*3*nx)

!  endsubroutine

!  subroutine snap_op2(sigmaxx,S3,S4)
!    integer                          :: wstat,srite
!    real,dimension(nz,nx),intent(in) :: sigmaxx,S3,S4

!    temparray(:,1:nx) = sigmaxx(:,:)/(maxval(sigmaxx(:,:))+1)
!    temparray(:,nx+1:2*nx) = S3(:,:)/(maxval(S3(:,:))+1)
!    temparray(:,2*nx+1:3*nx) = S4(:,:)/(maxval(S4(:,:))+1)

!    wstat = srite(fname,temparray,4*nz*3*nx)

!  endsubroutine

  subroutine snap_op3(sigmaxx)
    integer                          :: wstat,srite
    real,dimension(nz,nx),intent(in) :: sigmaxx

    temparray(:,:) = sigmaxx(:,:)

    wstat = srite("dat/sigmaxx_born.H",temparray,4*nz*nx)

  endsubroutine

  subroutine snap_op4(vx,vz,sigmaxx,sigmazz,sigmaxz)
    integer                          :: wstat,srite
    real,dimension(nz,nx),intent(in) :: vx,vz,sigmaxx,sigmazz,sigmaxz

    temparray(:,:) = vx(:,:)
    wstat = srite("dat/vx_bkg.H",temparray,4*nz*nx)
    temparray(:,:) = vz(:,:)
    wstat = srite("dat/vz_bkg.H",temparray,4*nz*nx)
    temparray(:,:) = sigmaxx(:,:)
    wstat = srite("dat/sigmaxx_bkg.H",temparray,4*nz*nx)
    temparray(:,:) = sigmazz(:,:)
    wstat = srite("dat/sigmazz_bkg.H",temparray,4*nz*nx)
    temparray(:,:) = sigmaxz(:,:)
    wstat = srite("dat/sigmaxz_bkg.H",temparray,4*nz*nx)
  endsubroutine

  subroutine snap_op5(vx,vz,sigmaxx,sigmazz,sigmaxz)
    integer                          :: wstat,srite
    real,dimension(nz,nx),intent(in) :: vx,vz,sigmaxx,sigmazz,sigmaxz

    temparray(:,:) = vx(:,:)
    wstat = srite("dat/vx_rtm.H",temparray,4*nz*nx)
    temparray(:,:) = vz(:,:)
    wstat = srite("dat/vz_rtm.H",temparray,4*nz*nx)
    temparray(:,:) = sigmaxx(:,:)
    wstat = srite("dat/sigmaxx_rtm.H",temparray,4*nz*nx)
    temparray(:,:) = sigmazz(:,:)
    wstat = srite("dat/sigmazz_rtm.H",temparray,4*nz*nx)
    temparray(:,:) = sigmaxz(:,:)
    wstat = srite("dat/sigmaxz_rtm.H",temparray,4*nz*nx)
  endsubroutine

endmodule

