! Program created by Gustavo Catao Alves at
! the Stanford Exploration Project
! May, 6th, 2014

! source module
module source_mod

  use sep

  implicit none
  real, private      :: fp, dt, dx
  integer, private   :: nx, nz, it, nt

  contains

  ! Initialization of all variables needed by this subroutine
  subroutine source_init(fp_in,dt_in, dx_in, nz_in, nx_in, nt_in)
    real                :: fp_in, dt_in, dx_in
    integer             :: nx_in, nz_in, nt_in

    fp = fp_in
    dt = dt_in
    dx = dx_in
    nz = nz_in
    nx = nx_in
    nt = nt_in
  end subroutine

  ! Ricker source
  subroutine source_op2(source)
    real                :: pi, aux, t, nsource
    integer             :: it
    real,dimension(nt)  :: source

    source=0.0
    pi = 3.14159265
    nsource=1.0/(sqrt(2.0)*pi*fp*dt)
    do it=1,nt
      t = (1.0*it-4.0*nsource)*dt
      aux = (pi*pi*fp*fp*t*t)
      source(it) = 10.0e0*(1.0 - 2.0*aux)*exp(-aux)
    enddo

    ! call to_history("n1", nt, "dat/source.H")
    ! call to_history("o1", 0., "dat/source.H")
    ! call to_history("d1", dt, "dat/source.H")
    ! call sep_write(source,"dat/source.H")

  endsubroutine

  ! Distribution of point sources to create an explosive source
  subroutine explosive_source(add,source,xsource,zsource,data)
    real                     :: source
    integer                    :: xsource,zsource
    real,dimension(nz,nx)    :: data
    logical                    :: add

    if (.not.add) data = 0.
    !write(0,*)maxval(data)
    data(zsource,xsource) = data(zsource,xsource) + dt*source
  endsubroutine

endmodule
